/*
 *  This file if part of nzbget
 *
 *  Copyright (C) 2004  Sven Henkel <sidddy@users.sourceforge.net>
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 * $Revision: 1.2 $
 * $Date: 2004/07/28 17:24:08 $
 *
 */

#include <stdlib.h>
#include <stdio.h>
#include <unistd.h>
#include <stdarg.h>

#include "ColoredFrontend.h"

#include "NZBQueueCoordinator.h"

extern int threadCount;
extern NZBQueueCoordinator* g_pNZBQueueCoordinator;

ColoredFrontend::ColoredFrontend()
{
}

ColoredFrontend::~ColoredFrontend()
{
}

void ColoredFrontend::frontendError(char* msg)
{
  char tmp[1024];

  snprintf(tmp, 1024, "\033[31m[ERROR]\033[39m %s\033[K\n", msg);

  pthread_mutex_lock(&this->mutex_messages);

  this->messages.push_back(strdup(tmp));

  pthread_mutex_unlock(&this->mutex_messages);
}

void ColoredFrontend::frontendWarn(char* msg)
{
  char tmp[1024];

  snprintf(tmp, 1024, "\033[35m[WARNING]\033[39m %s\033[K\n", msg);

  pthread_mutex_lock(&this->mutex_messages);

  this->messages.push_back(strdup(tmp));

  pthread_mutex_unlock(&this->mutex_messages);
}

void ColoredFrontend::frontendDebug(char* msg)
{
  char tmp[1024];

  snprintf(tmp, 1024, "[DEBUG] %s\033[K\n", msg);

  pthread_mutex_lock(&this->mutex_messages);

  this->messages.push_back(strdup(tmp));

  pthread_mutex_unlock(&this->mutex_messages);
}

void ColoredFrontend::frontendInfo(char* msg)
{
  char tmp[1024];

  snprintf(tmp, 1024, "\033[32m[INFO]\033[39m %s\033[K\n", msg);

  pthread_mutex_lock(&this->mutex_messages);

  this->messages.push_back(strdup(tmp));

  pthread_mutex_unlock(&this->mutex_messages);
}

void ColoredFrontend::frontendQueueListEntry( const char* msg )
{
	char tmp[1024];
	
	snprintf(tmp, 1024, "\033[32m[ENTRY]\033[39m %s\033[K\n", msg);
	
	pthread_mutex_lock(&this->mutex_messages);
	
	this->messages.push_back(strdup(tmp));
	
	pthread_mutex_unlock(&this->mutex_messages);
}

void ColoredFrontend::printStatus()
{
	char tmp[1024];

	pthread_mutex_lock(&this->mutex_messages);

	int lines = 0;

	fprintf(stdout, "\033[s");
	
	float fCurrentDownloadSpeed = g_pNZBQueueCoordinator->CalcCurrentDownloadSpeed();
  
	char timeString[100];
	
	timeString[0] = '\0';
	
	if( fCurrentDownloadSpeed > 0.0f )
	{
		long long remain_sec = this->queueSize / ((long long int)( fCurrentDownloadSpeed * 1024));
		int h = 0;
		int m = 0;
		int s = 0;
		while (remain_sec > 3600)
		{
			h++;
			remain_sec -= 3600;
		}
		
		while (remain_sec > 60)
		{
			m++;
			remain_sec -= 60;
		}
		
		s = remain_sec;
		
		sprintf(timeString, "~ %.2d:%.2d:%.2d", h, m, s);
	}

      

	snprintf(tmp, 1024, "%d threads running, %.0f kB/s, %d MB queued (%s)\033[K\n",
		threadCount, fCurrentDownloadSpeed, (int) (this->queueSize / 1024 / 1024), timeString);
	fprintf(stdout, "%s", tmp);
	lines++;

	// add message to go back some lines
	
	snprintf(tmp, 1024, "\r\033[%dA", lines);
	
	this->messages.push_back(strdup(tmp));
	
	pthread_mutex_unlock(&this->mutex_messages);
	
	fflush(stdout);
}

