/*
 *  This file if part of nzbget
 *
 *  Copyright (C) 2004  Sven Henkel <sidddy@users.sourceforge.net>
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 * $Revision: 1.10 $
 * $Date: 2004/07/28 17:24:08 $
 *
 */

#include <stdlib.h>
#include <stdio.h>
#include <unistd.h>
#include <stdarg.h>

#include "Frontend.h"

extern int threadCount;

Frontend::Frontend()
{
  this->messages.clear();
  this->queueSize = 0;

  pthread_mutex_init(&mutex_messages, NULL);
}

Frontend::~Frontend()
{
}

void Frontend::frontendError(char* msg)
{
  char tmp[1024];

  snprintf(tmp, 1024, "[ERROR] %s\n", msg);

  pthread_mutex_lock(&this->mutex_messages);

  this->messages.push_back(strdup(tmp));

  pthread_mutex_unlock(&this->mutex_messages);
}

void Frontend::frontendWarn(char* msg)
{
  char tmp[1024];

  snprintf(tmp, 1024, "[WARNING] %s\n", msg);

  pthread_mutex_lock(&this->mutex_messages);

  this->messages.push_back(strdup(tmp));

  pthread_mutex_unlock(&this->mutex_messages);
}

void Frontend::frontendDebug(char* msg)
{
  char tmp[1024];

  snprintf(tmp, 1024, "[DEBUG] %s\n", msg);

  pthread_mutex_lock(&this->mutex_messages);

  this->messages.push_back(strdup(tmp));

  pthread_mutex_unlock(&this->mutex_messages);
}

void Frontend::frontendInfo(char* msg)
{
  char tmp[1024];

  snprintf(tmp, 1024, "[INFO] %s\n", msg);

  pthread_mutex_lock(&this->mutex_messages);

  this->messages.push_back(strdup(tmp));

  pthread_mutex_unlock(&this->mutex_messages);
}

void Frontend::run()
{
	while(1)
	{
		printMessages();
		usleep( (int)GetSleepTime() );
	}
}

void Frontend::printMessages()
{
  pthread_mutex_lock(&this->mutex_messages);

  int newline = 0;
  
  for (unsigned int i = 0; i < this->messages.size(); i++)
    {
      fprintf(stdout, "%s", this->messages[i]);
      free(this->messages[i]);
      this->messages[i] = NULL;
      newline = 1;
    }

  this->messages.clear();

  pthread_mutex_unlock(&this->mutex_messages);

  if (newline == 1)
    this->printStatus();
}

void Frontend::printStatus()
{
}

void Frontend::frontendQueueListEntry( const char* szEntry )
{
	char tmp[1024];
	
	snprintf(tmp, 1024, "[ENTRY] %s\n", szEntry);
	
	pthread_mutex_lock(&this->mutex_messages);
	
	this->messages.push_back(strdup(tmp));
	
	pthread_mutex_unlock(&this->mutex_messages);
}
