/*
 *  This file if part of nzbget
 *
 *  Copyright (C) 2005  Bo Cordes Petersen <placebodk@sourceforge.net>
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 * $Revision: 1.8 $
 * $Date: 2004/07/14 18:47:14 $
 *
 */

#include "NZBQueueMessageControl.h"
#include "NZBQueueCoordinator.h"
#include "Options.h"
#include "Frontend.h"
#include "unistd.h"

extern Frontend* frontend;
extern Options* options;

NZBQueueMessageControl::NZBQueueMessageControl()
{
	const char* szNetAddress = options->getOption( OPTION_SERVERIP, NULL );
	m_iPort = atoi( options->getOption( OPTION_SERVERPORT, NULL ) );
	m_szPassword = options->getOption( OPTION_SERVERPASSWORD, NULL );
	m_pNetAddress = new NetAddress( szNetAddress, m_iPort );
	m_pConnection = new Connection( m_pNetAddress );

	m_pConnection->Bind();
}

NZBQueueMessageControl::~NZBQueueMessageControl()
{
}

void NZBQueueMessageControl::run()
{
	char tmp[1024];

	while( true )
	{
		int iSocket;
		// Accept connections and store the "new" socket value
		if( (iSocket = m_pConnection->Accept()) == -1 )
		{
			continue;
		}

		int iResult = 0;

		// Read the first package which needs to be a request
		
		SNZBQueueRequest Request;
		iResult = m_pConnection->Recv( iSocket, (char*)&Request, sizeof( Request )  );
		if( iResult < 0 )
		{
			continue;
		}

		// Make sure this is a nzbget request from a client
		if( Request.m_iId != 'nzbg' )
		{
			sprintf( tmp, "Non-nzbget request received on port %i", m_iPort );
			frontend->frontendInfo( tmp );

			if( iSocket > -1 )
			{
			  close( iSocket );
			}
			
			continue;
		}

		if( strcmp( Request.m_szPassword, m_szPassword ) )
		{
		sprintf( tmp, "nzbget request received on port %i, but password invalid", m_iPort );
			frontend->frontendInfo( tmp );

			if( iSocket > -1 )
			{
			  close( iSocket );
			}
			
			continue;
		}

		// Info - connection received
		struct sockaddr_in PeerName;
		int iPeerNameLength = sizeof( PeerName );

		if( getpeername( iSocket, (struct sockaddr*)&PeerName, (socklen_t*) &iPeerNameLength) >= 0 )
		{
			sprintf( tmp, "NZBQueue request received from %s", inet_ntoa( PeerName.sin_addr) );
			frontend->frontendInfo( tmp );
		}
		
		int iCurrentSize = CONNECTION_BUFFER_SIZE;
		char* pRecvBuffer = NULL;
		char* pOldRecvBuffer = NULL;
		
		switch( Request.m_iType )
		{
			case NZBMessageRequest::eNewDownload:
						
				pRecvBuffer = new char[ iCurrentSize ];
				memset( pRecvBuffer, 0, iCurrentSize ); // Clear the buffer
			
				// Read from the socket until nothing remains
				do
				{
					iResult = m_pConnection->Recv( iSocket );
		
					// Did the recv succeed?
					if( iResult > 0 )
					{
						int iLen = strlen( m_pConnection->GetReadBuffer() ) + strlen( pRecvBuffer );

/*						TEST
						FILE* pFileConnection = fopen( "connection.txt", "a+" );
						if( pFileConnection )
						{
							fprintf( pFileConnection, "\n---------------[ NEW RECEIVE ]-----------\n" );
							fprintf( pFileConnection, "%s", m_pConnection->GetReadBuffer() );
							fclose( pFileConnection );
						}
						else
						{
							perror( "fopen" );
						}
*/
						if( iLen > iCurrentSize )
						{
							pOldRecvBuffer = pRecvBuffer;
							int iNewLength = iCurrentSize << 2;
							pRecvBuffer = new char[ iNewLength ];
							memset( pRecvBuffer, 0, iNewLength );
							memcpy( pRecvBuffer, pOldRecvBuffer, iCurrentSize );
							delete pOldRecvBuffer;
							iCurrentSize = iNewLength;
						}
									
						strcat( pRecvBuffer, m_pConnection->GetReadBuffer() );
					}
					else if( iResult < 0 )
					{
						if( pRecvBuffer )
						{
							delete pRecvBuffer;
							pRecvBuffer = NULL;
						}
						
						break;;
					}
					else
					{
						break;
					}
				}
				while( iResult != 0 );

				if( pRecvBuffer )
				{
					
					/* TEST
					printf( "pRecvBuffer size %i\n", strlen( pRecvBuffer ), iCurrentSize );

					FILE* pFile = fopen( "received.txt", "a+" );

					if( !pFile )
					{
					perror( "fopen" );
				}
					else
					{
					fprintf( pFile, "\n---------------[ NEW RECEIVE ]-----------\n" );
					fprintf( pFile, "%s", pRecvBuffer );
					fclose( pFile );
				}
					*/
				
					NZBQueue* pQueue = m_pQueueCoordinator->CreateNZBQueueFromBuffer( Request.m_szFileName, pRecvBuffer, iCurrentSize );

					if( pQueue )
					{
						sprintf( tmp, "Request: Queue file %s", Request.m_szFileName );
						frontend->frontendInfo( tmp );

						m_pQueueCoordinator->AddNZBQueueToDownloadQueue( pQueue, true );

						// Currently doesnt seem to work
/*
						char tmp[1024];
						sprintf( tmp, "NZBFile %s added to queue", Request.m_szFileName );
						m_pConnection->Send( iSocket, tmp, strlen( tmp ) );
*/
					}

					delete pRecvBuffer;
				}
				
				break;
			case NZBMessageRequest::ePause:
			case NZBMessageRequest::eUnPause:
			case NZBMessageRequest::eListDownloads:
#ifdef DEBUG
				printf( "NZB-Request not yet supported\n" );
#endif
				return;

		}
	}
}





