/*
 *  This file if part of nzbget
 *
 *  Copyright (C) 2004  Sven Henkel <sidddy@users.sourceforge.net>
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 * $Revision: 1.5 $
 * $Date: 2004/08/10 19:22:42 $
 *
 */

/*
 *  
 * Semaphore Patch by Florian Penzkofer <f.penzkofer@sent.com>
 * The queue of mutexes that was used did not work for every
 * implementation of POSIX threads. Now a Semaphore is used.
 *   
 */


#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#include <unistd.h>
#include <semaphore.h>

#include "SingleServerPool.h"
#include "global.h"

SingleServerPool::SingleServerPool(NewsServer* s, int max)
{
	debug("SingleServerPool::SingleServerPool(NewsServer*, int)\n");
	this->server = s;
	this->max = max;
	this->id = 0;
	this->connections.clear();
	this->free.clear();
	
	sem_init(&semaphore, 0, max);
	pthread_mutex_init(&free_mutex, NULL);
	
	for (int i = 0; i < this->max; i++)
	{
		this->connections.push_back(new NNTPConnection(this->server));
		this->free.push_back(i);
	}
};

int SingleServerPool::getID()
{
  debug("SingleServerPool::getID() : Assigning ID %d\n",id+1);
  return ++id;
}

NNTPConnection* SingleServerPool::getConnection()
{
	int connection_nr = -1;
		
	// decrease semaphore counter or block
	sem_wait(&semaphore);
	
	pthread_mutex_lock(&free_mutex);

	if (free.size() > 0)
	{
		// free connection found, take it!
		connection_nr = free.front();
		free.pop_front();
	}
	
	pthread_mutex_unlock(&free_mutex);
		
	if (connection_nr == -1)
	{
		fprintf(stderr, "SingleServerPool: serious error, no free connection found, but there should be one.\n");
		return NULL;
	}
	
	return connections[connection_nr];
}

void SingleServerPool::freeConnection(NNTPConnection* con)
{
    // give back free connection
    for (int i = 0; i < this->max; i++)
    {
		if (connections[i] == con)
		{
			debug("SingleServerPool::freeConnection(NNTPConnection*) : Connection %d freed\n",i);
 		
			pthread_mutex_lock(&free_mutex);
			free.push_back(i);
			pthread_mutex_unlock(&free_mutex);
			
			sem_post(&semaphore);

			return;
		}	
    }

	fprintf( stderr, "SingleServerPool::freeConnection\n" );
	fprintf( stderr, "Something screwy happened here - trying to free a connection that could not be found in the list\n" );
}